<?php

require_once("lib/cookie.class.php");
require_once("lib/groups.class.php");

class common
{
    private $db, $cookie;
    public $conf, $user_conf, $lang, $groups, $logged_in;
    
    function __construct()
    {       
        $this->conf=parse_ini_file("etc/wiki.conf", true);
        $this->lang=parse_ini_file("etc/".$this->conf[main][language].
                                   ".conf", true);
        $this->db = new SQLiteDatabase( $this->conf[db][file] );
        $this->cookie = new cookie($conf[cookie]);
        $this->groups = new groups($this->db);
        
        /* test if the user is logged in (so we don't do it 4x+ times) */
        $this->logged_in = $this->is_logged_in();

        // Remove the cookie
        if ($_GET['erasecookie'] == 1)
            $this->logout();

        $this->get_user_conf($this->logged_in);
    }
    
    function __destruct()
    {
        
    }
    
    function output_page( $action, $title, $content, $time, $error = "" )
    {
        /* Note: a small "renderer", content should be pre-"rendered" */

        /* include the config and read the template */

        if( ! $file = @fopen( "template.html", "r" ))
            die( "Error: unable to read 'template.html'" );
        $html = fread( $file, filesize( "template.html" ));
        fclose( $file );
        
        /* check if $error is set */
        if( $error == "" )
            $html = str_replace( '{ERROR}', "", $html );
        else
            $html = str_replace( '{ERROR}', $error, $html );

        /* menu rendering */
        if ( $title == $START_PAGE )
            $html = str_replace( '{HOME}', $this->lang[menu][start], $html );
        else
            $html = str_replace( '{HOME}', "<a href=\"./?page=".$this->lang[header][start_page]."\">".$this->lang[menu][start]."</a>", $html );

        if( $action == "tags" )
            $html = str_replace('{TAGS}', $this->lang[menu][tags], $html);
        else
            $html = str_replace('{TAGS}', "<a href=\"./?action=tags\">".$this->lang[menu][tags]."</a>", $html);

        if( $action == "recent" )
            $html = str_replace('{RECENT_CHANGES}', $this->lang[menu][recent_changes], $html);
        else
            $html = str_replace('{RECENT_CHANGES}', "<a href=\"./?action=recent\">".$this->lang[menu][recent_changes]."</a>", $html);
        
        if( $action == "edit" || $action == "search" || $action == "recent" || ! $this->cookie->ok($this->db) ||
            $action == "img_up" || $action == "tags" || $action == "settings" )
            $html = str_replace( '{EDIT}', $this->lang[menu][edit], $html );
        elseif (is_writable( $this->conf[folders][pages] . $title . ".txt") || !file_exists($this->conf[folders][pages] . $title . ".txt" ))
            $html = str_replace( '{EDIT}', "<a href=\"./?page=".$title."&amp;action=edit\">".$this->lang[menu][edit]."</a>", $html );
        else
            $html = str_replace( '{EDIT}', $this->lang[menu]['protected'], $html );
        
        if ( $action == "edit") {
            $html = str_replace( '{HELP}', "(<a href=\"./?page=".$this->lang[menu][help]."\">".$this->lang[menu][help]."</a>)", $html );
            $html = str_replace( '{HISTORY}', "/ <a href=\"?page=".$title."&amp;action=history\">".$this->lang[menu][history]."</a>", $html );
            $html = str_replace( '{IMG_UL}', "/ <a href=\"/img_up.php\">".$this->lang[menu][image_ul]."</a>", $html );
            $html = str_replace( '{GALLERY}', "/ <a href=\"/".$this->conf[folders][image_targ]."\">".$this->lang[menu][gallery]."</a><br />", $html );
        }
        else
        {
            $html = str_replace( '{HELP}', "", $html );
            $html = str_replace( '{HISTORY}', "", $html );
            $html = str_replace( '{IMG_UL}', "", $html );
            $html = str_replace( '{GALLERY}', "<br />", $html );
        }

        /* login / erase cookie */
        if ( $this->cookie->ok($this->db) )
        {
            $html = str_replace( '{COOKIE}', '<a href="/?page='.$title.'&erasecookie=1">'.$this->lang[menu][logout] . $this->get_user() .'</a>', $html );
            if( $action != "settings" )
                $html = str_replace( '{SETTINGS}', '<a href="/settings.php">'.$this->lang[menu][settings] .'</a>', $html );
            else
                $html = str_replace( '{SETTINGS}', $this->lang[menu][settings], $html );
        }
        else
        {
            $html = str_replace( '{COOKIE}', '<a href="/login.php">'.$this->lang[menu][login].'</a>', $html );
            $html = str_replace( '{SETTINGS}', '', $html );
        }
        
        /* search field */
        $html = str_replace('{SEARCH}', "<form method=\"get\" action=\"./?page=".$_GET[page]."\"><div><input type=\"hidden\" name=\"action\" value=\"search\" /><input type=\"text\" name=\"query\" value=\"$_GET[query]\" class=\"inputbox\" /> <input type=\"submit\" value=\"".$this->lang[forms][search]."\" class=\"inputbox\" /></div></form>", $html);
        
        /* some misc varables */
        $html = str_replace( '{PAGE_TITLE_BRUT}', $title, $html );
        if( $action == "edit" || $action == "history" )
            $html = str_replace('{PAGE_TITLE}', "<a href=\"./?page=".$title."\">".$title."</a>", $html);
        else
            $html = str_replace( '{PAGE_TITLE}', $title, $html );
        $date = strftime( $this->conf[main][time_format] );
        $html = str_replace( '{DATE}', $date, $html );
        $html = str_replace( '{IP}', $_SERVER['REMOTE_ADDR'], $html );
        $html = str_replace( "{WIKI_VERSION}", $this->conf[misc][version], $html );
        $html = str_replace( '{WIKI_TITLE}', $this->conf[main][title], $html );
        $html = str_replace( '{LAST_CHANGE}', $this->lang[header][last_modified]." :", $html );
        $html = str_replace( '{TIME}', $time, $html );
        $html = str_replace( '{CHARSET}', $this->conf[main][charset], $html );
        $html = str_replace( '{CONTENT}', $content, $html);
        $html = str_replace( '{STYLE_SHEET}', $this->conf[folders][css]."/".$this->user_conf[main][css], $html);
        echo "$html";
    }
    
    /* Login functions */
    
    public function login( $user, $password )
    {
        /* check in db if user exists and if its password is correct */
        $result = $this->db->query( "SELECT * FROM users WHERE name='" . $this->escape($user) . "'" );
        $ok = False;

        $password = $this->hash($password);

        while( $result->valid() )
        {
            $row = $result->current();
            
            if( $row[name] == $user )
            {
                if( $row[password] == "" ) /* Locked */
                    return False;

                if( $password == $row[password] )
                    $ok = True;
                break;
            }
            $result->next();
        }
        /* give cookie */
        if( $ok )
        {
            $this->cookie->create($row[id], $password);
        }

        return $ok;
    }
    
    private function is_logged_in()
    {
        return $this->cookie->ok($this->db);
    }
    
    function logout()
    {
        $this->cookie->kill();
        header( 'Location: /' );
        die();
    }
    
    /* User handling functions */
    
    function get_user()
    {
        /* Return the username */
        $result = $this->db->query( "SELECT name FROM users WHERE id='" .
                              $this->escape($_COOKIE["KakAbuser"]) . "' LIMIT 1" );

        if( $result->valid() )
        {
            $row = $result->current();
            return $row[name];
        }

        $this->logout(); /* Logout so nothing worse can happen */
        return "misingo!";
    }
    
    function get_user_conf($logged_in)
    {
        if($logged_in){
            $sql = "SELECT css FROM users WHERE id='".
            $this->escape($_COOKIE["KakAbuser"])."'";
            $result = $this->db->query($sql);
            if($result->valid())
            {
                $row = $result->current();
                $this->user_conf[main][css] = $row['css'];
            }
            else
                return false;
        }
        else
        {
            $this->user_conf[main][css] = $this->conf[main][css];
        }
        
    return true;
    }
    
    function change_password( $old, $new )
    {
        /*
         first checks if uid is correct and old pwd, then we update.
        */
        
        if( $new == "" )
            return False;

        $old = $this->hash( $old );
        $new = $this->hash( $new );

        $result = $this->db->query( "SELECT * FROM users WHERE id='" .
                              $this->escape($_COOKIE["KakAbuser"]) . "' LIMIT 1" );

        if( $result->valid() )
        {
            $row = $result->current();
            if( $row[password] == $old and
                $this->cookie->hash( $row[password]) == $_COOKIE["Magic"] )
            {
                $this->db->query( 'UPDATE users SET password="'. $new
                            .'" WHERE id="'. $this->escape($_COOKIE["KakAbuser"]) .'"' );
                return True;
            }
            else
                return False;
        }

        $this->logout(); /* Logout so nothing worse can happen */
    }
    
    function set_password( $user, $password )
    {
        /* Just sets the password for uid */
        
        if( $password != "" )
            $password = $this->hash( $password );

        $this->db->query( 'UPDATE users SET password="'.
                    $password .'" WHERE id="'. $this->escape($user) .'"' );
    }
    
    function set_css( $css_file )
    {
        /* set the css file after checking that the users isn't bluffing */
        
        if( $css_file == "" )
            return False;

        $css_file = $this->escape( $css_file );

        $result = $this->db->query( "SELECT * FROM users WHERE id='" .
                                    $this->escape($_COOKIE["KakAbuser"])
                                    . "' LIMIT 1" );

        if( $result->valid() )
        {
            $row = $result->current();
            if( $this->cookie->hash( $row[password]) == $_COOKIE["Magic"] )
            {
                $this->db->query( 'UPDATE users SET css="'. $css_file
                            .'" WHERE id="'.
                                  $this->escape($_COOKIE["KakAbuser"]) .'"' );
                return True;
            }
            else
                return False;
        }
    }

    function new_user( $name, $password )
    {
        /* returns true if no problems where found */
        $password = $this->hash( $password );
        $result = $this->db->query( "SELECT id FROM users WHERE name='" .
                              $this->escape($name) . "' LIMIT 1" );

        if( $result->valid() )
            return False;

        $this->db->query( 'INSERT INTO users ("name", "password", "css") VALUES( "'
                    .$this->escape($name).'", "'. $password .'", "'.$this->conf[main][css].'")' );

        $result = $this->db->query( "SELECT id FROM users WHERE name='" .
                              $this->escape($name) . "' LIMIT 1" );

        if( $result->valid() )
        {
            $row = $result->current();
            $this->cookie->create($row[id], $password);

            return True;
        }
        return False;
    }
    
    /* Support functions */
    
    //Hash function for db, for cookie in cookie class
    private function hash($password)
    {
        return md5( $password . $this->conf[db][salt]);
    }
    
    private function escape($string)
    {
        //WARNING this function is used separatly in other places!
        //E.g. cookie.class and groups.class
        return sqlite_escape_string($string);
    }
}

/*
 Known missing functions:
    None!
*/

/* SQL defines
 create table users(
    id INTEGER PRIMARY KEY,
    name VARCHAR,
    password VARCHAR
    css VARCHAR
 );
 · One could belong to many groups
 create table groups(
    uid int,
    groupname VARCHAR
 );

*/

?>
